// $Date:: 2009-08-23 19:41:38 +0900#$
// $Rev: 2 $

#include "MoveChroma.h"

/**************************************************************************************************/
/* MoveChroma                                                                                      */
/**************************************************************************************************/
//==============================================================================
// constructor
//==============================================================================
MoveChroma::MoveChroma(PClip _child, int cb, int cr, IScriptEnvironment* env) : GenericVideoFilter(_child)
{
    this->Height    = this->vi.height ;
    this->Width     = this->vi.width ;

    if ( abs ( cb ) > this->Width )
        env->ThrowError("Cb should be smoller than Width.");

    if ( abs ( cr ) > this->Width )
        env->ThrowError("Cr should be smoller than Width.");

    this->Cb = cb ;
    this->Cr = cr ;
}

//==============================================================================
// GetFrame
//==============================================================================
PVideoFrame MoveChromaYV12::GetFrame (int n, IScriptEnvironment* env)
{
    PVideoFrame src = this->child->GetFrame ( n , env ) ;
    env->MakeWritable ( & src ) ;

	BYTE* pSrcU = src->GetWritePtr(PLANAR_U);
	BYTE* pSrcV = src->GetWritePtr(PLANAR_V);
	const unsigned int nSrcUPitch = src->GetPitch(PLANAR_U);
	const unsigned int nSrcVPitch = src->GetPitch(PLANAR_V);

    this->Move ( pSrcU , nSrcUPitch , this->Cb ) ;
    this->Move ( pSrcV , nSrcVPitch , this->Cr ) ;

    return src ;
}

void MoveChromaYV12::Move ( BYTE* pSrc , const unsigned int nSrcPitch , const int nMove )
{
    if ( nMove == 0 ) return ;

    BYTE * pDst = pSrc ;
    const int d0 = nMove / 2 , d1 =( nMove - 1 ) / 2 , w = this->Width / 2 - 1 , h = this->Height / 2 ;

    if ( nMove < 0 )
    {
        for ( int j = 0 ; j < h ; j ++ , pDst += nSrcPitch , pSrc += nSrcPitch )
        {
            for ( int i = 0 ; i < w + d0 ; i ++ )
                pDst [ i ] = ( pSrc [ i - d0 ] + pSrc [ i - d1 ] ) / 2 ;

            for ( int i = w + d0 ; i < w ; i ++ )
                pDst [ i ] = pSrc [ w ] ;
        }
    }
    else
    {
        for ( int j = 0 ; j < h ; j ++ , pDst += nSrcPitch , pSrc += nSrcPitch )
        {
            for ( int i = w ; i > d0 ; i -- )
                pDst [ i ] = ( pSrc [ i - d0 ] + pSrc [ i - d1 ] ) / 2 ;

            for ( int i = d0 ; i > 0 ; i -- )
                pDst [ i ] = pSrc [ 0 ] ;
        }
    }
}


//==============================================================================
// GetFrame
//==============================================================================
PVideoFrame MoveChromaYUY2::GetFrame (int n, IScriptEnvironment* env)
{
    PVideoFrame src = this->child->GetFrame ( n , env ) ;
    env->MakeWritable ( & src ) ;

	BYTE* pSrc = src->GetWritePtr();
	const unsigned int nSrcPitch = src->GetPitch();

    this->Move ( pSrc + 1 , nSrcPitch , this->Cb ) ;
    this->Move ( pSrc + 3 , nSrcPitch , this->Cr ) ;

    return src ;
}

void MoveChromaYUY2::Move ( BYTE* pSrc , const unsigned int nSrcPitch , const int nMove )
{
    if ( nMove == 0 ) return ;

    BYTE * pDst = pSrc ;
    const int d0 = nMove / 2 * 4 , d1 = ( nMove - 1 ) / 2 * 4 , w = this->Width * 2 - 4 , h = this->Height ;

    if ( nMove < 0 )
    {
        for ( int j = 0 ; j < h ; j ++ , pDst += nSrcPitch , pSrc += nSrcPitch )
        {
            for ( int i = 0 ; i < w + d0 ; i += 4 )
                pDst [ i ] = ( pSrc [ i - d0 ] + pSrc [ i - d1 ] ) / 2 ;

            for ( int i = w + d0 ; i < w ; i += 4 )
                pDst [ i ] = pSrc [ w ] ;
        }
    }
    else
    {
        for ( int j = 0 ; j < h ; j ++ , pDst += nSrcPitch , pSrc += nSrcPitch )
        {
            for ( int i = w ; i > d0 ; i -= 4 )
                pDst [ i ] = ( pSrc [ i - d0 ] + pSrc [ i - d1 ] ) / 2 ;

            for ( int i = d0 ; i > 0 ; i -= 4 )
                pDst [ i ] = pSrc [ 0 ] ;
        }
    }
}
